@@ -2,10 +2,10 @@ package Devel::Cover::Report::Html_subtle;
 use strict;
 use warnings;
 
-our $VERSION = "0.22";
+our $VERSION = "0.23";
 
-use Devel::Cover::DB 0.22;
-use Devel::Cover::Truth_Table 0.22;
+use Devel::Cover::DB 0.23;
+use Devel::Cover::Truth_Table 0.23;
 
 use Cwd();
 use Template 2.00;
@@ -79,6 +79,9 @@ sub print_summary {
 					elsif ($criterion eq 'condition') {
 						$vals{$file}{$criterion}{link} = "$Filenames{$file}--condition.html";
 					}
+					elsif ($criterion eq 'subroutine') {
+						$vals{$file}{$criterion}{link} = "$Filenames{$file}--subroutine.html";
+					}
 				}
 			}
 			$vals{$file}{$criterion}{pc} = $pc;
@@ -177,6 +180,15 @@ sub print_file {
 				}
 				push @{$line{metrics}}, \@p;
 			}
+			elsif ($c eq 'subroutine') {
+			    my @p;
+				while (my $o = shift @{$metric{$c}}) {
+					push @p, {text  => $o->covered,
+							  class => $o->error ? 'uncovered' : 'covered',
+							  link  => "$Filenames{$file}--subroutine.html#line$."};
+				}
+				push @{$line{metrics}}, \@p;
+			}
 			else {
 			    my @p;
 				while (my $o = shift @{$metric{$c}}) {
@@ -298,6 +310,41 @@ sub print_conditions {
 	  or die $Template->error();
 }
 
+sub print_subroutines {
+	my ($db, $file, $options) = @_;
+	my $subroutines = $db->cover->file($file)->subroutine;
+	return unless $subroutines;
+
+    my @data;
+    for my $location ($subroutines->items)
+    {
+        my $l = $subroutines->location($location);
+        for my $sub (@$l)
+        {
+            push @data, {
+                            ref   => "line$location",
+                            line  => $location,
+                            name  => $sub->name,
+                            class => cvg_class($sub->percentage),
+                        }
+        }
+    }
+
+	my $vars = {
+		title      => "Subroutine Coverage: $file",
+		file        => $file,
+		percentage  => sprintf("%.1f", $db->{summary}{$file}{subroutine}{percentage}),
+		class       => cvg_class($db->{summary}{$file}{subroutine}{percentage}),
+		subroutines =>  [ sort { $a->{name} cmp $b->{name} } @data ],
+		perlver     => join('.', map {ord} split(//, $^V)), # should come from db
+		platform    => $^O,                                 # should come from db
+	};
+
+	my $html = "$db->{db}/$Filenames{$file}--subroutine.html";
+	$Template->process("subroutines", $vars, $html)
+	  or die $Template->error();
+}
+
 
 #-------------------------------------------------------------------------------
 # Subroutine : report()
@@ -318,9 +365,10 @@ sub report {
 	print_summary($db, $options);
 
 	for my $file (@{$options->{file}}) {
-		print_file($db,       $file, $options);
-		print_branches($db,   $file, $options) if $options->{show}{branch};
-		print_conditions($db, $file, $options) if $options->{show}{condition};
+		print_file($db,        $file, $options);
+		print_branches($db,    $file, $options) if $options->{show}{branch};
+		print_conditions($db,  $file, $options) if $options->{show}{condition};
+		print_subroutines($db, $file, $options) if $options->{show}{subroutine};
 	}
 }
 
@@ -330,7 +378,7 @@ package Devel::Cover::Report::Html_subtle::Template::Provider;
 use strict;
 use warnings;
 
-our $VERSION = "0.22";
+our $VERSION = "0.23";
 
 use base "Template::Provider";
 
@@ -348,7 +396,7 @@ sub fetch {
 $Templates{html} = <<'EOT';
 <!--
 
-This file was generated by Devel::Cover Version 0.22
+This file was generated by Devel::Cover Version 0.23
 
 Devel::Cover is copyright 2001-2002, Paul Johnson (pjcj@cpan.org)
 
@@ -536,6 +584,47 @@ $Templates{conditions} = <<'EOT';
 [% END %]
 EOT
 
+$Templates{subroutines} = <<'EOT';
+[% WRAPPER html %]
+
+<h1>Subroutine Coverage</h1>
+<table>
+	<tr>
+		<td class="header" align="right">File:</td>
+		<td>[% file %]</td>
+	</tr>
+	<tr>
+		<td class="header" align="right">Coverage:</td>
+		<td class="[% class %]">[% percentage %]%</td>
+	</tr>
+	<tr>
+		<td class="header" align="right">Perl version:</td>
+		<td>[% perlver %]</td>
+	</tr>
+	<tr>
+		<td class="header" align="right">Platform:</td>
+		<td>[% platform %]</td>
+	</tr>
+</table>
+<div><br></br></div>
+<table>
+    <tr valign="top">
+        <th class="header"> subroutine </th>
+        <th class="header"> line </th>
+    </tr>
+
+    [% FOREACH sub = subroutines %]
+        <tr align="center" valign="top">
+            <td class="[% sub.class %]"> <a id="[% sub.ref %]"> [% sub.name %] </td>
+            <td> [% sub.line %] </td>
+        </tr>
+    [% END %]
+
+</table>
+
+[% END %]
+EOT
+
 $Templates{file} = <<'EOT';
 [% WRAPPER html %]
 
@@ -633,7 +722,7 @@ Huh?
 
 =head1 VERSION
 
-Version 0.22 - 2nd September 2003
+Version 0.23 - 6th September 2003
 
 =head1 LICENCE
 
